from injector import singleton

from tracker.core.connectors.git.bitbucket import BitBucketHookParser
from tracker.core.connectors.issue_handler import IssueHandler
from tracker.core.connectors.webhook_parser import WebHookDataParser


def configure(binder):
    binder.bind(WebHookDataParser, to=BitBucketHookParser, scope=singleton)
    binder.bind(IssueHandler, to=IssueHandler, scope=singleton)




import logging
from abc import abstractmethod, ABC

from tracker.core.connectors.webhook_parser import RefChangeRequest

logger = logging.getLogger("issue:handler")


class IssueHandler(ABC):
    def __init__(self):
        pass

    def handle(self, issues, request: RefChangeRequest):
        if len(issues) > 0:
            self.process(issues, request)
        else:
            logger.info("issues not found")

    @abstractmethod
    def process(self, issues, request: RefChangeRequest):
        pass



import logging
import re
import shutil

import git
from git import Repo

from tracker.core.connectors.issue_handler import IssueHandler
from tracker.core.connectors.webhook_parser import RefChangeRequest
from tracker.env import GIT_USER_PASS, MERGE_PATTERN_SEARCH_TO_SKIP, ISSUE_TRACKER_PATTERN

logger = logging.getLogger("git:repository:process")
separator = "======="


def process_hook_data(request: RefChangeRequest, handler: IssueHandler):
    try:
        committed_issues = __find_merged_commits__(request)
        handler.handle(committed_issues, request)
    except Exception as e:
        logger.error("Error has happened either on processing ref change or post hook" + str(e))


def __find_merged_commits__(request: RefChangeRequest) -> [str]:
    repo_path = '/tmp/tracker_{}'.format(request.repo_name)

    def checkout_repo(path):
        r: Repo = None
        try:
            r = git.Repo(path)
        except Exception as e:
            logger.error(e)

        if r is None:
            logger.info("Cloning with GitPython over https with the username and token")
            try:
                r = git.Repo.clone_from("https://{}:{}@{}".format(
                    GIT_USER_PASS["username"],
                    GIT_USER_PASS["token"],
                    request.repo_link.split("https://")[1]), path)
            except Exception as e:
                logger.error(e)
        return r

    repo = checkout_repo(repo_path)
    try:
        logger.info("Fetch changes from remote")
        repo.git.fetch()
    except Exception as e:
        try:
            logger.info("Remove folder with project due to error in git tree {}".format(repo_path))
            shutil.rmtree(repo_path)
        except OSError as e:
            logger.error("Error: {} : {}".format(repo_path, e.strerror))
        checkout_repo(repo_path)

    included_issues = set()
    commits = repo.git.log('--pretty=Commit: %h Date: %ci%nAuthor: %ce%n%n%s%n%b%n{}'.format(separator),
                           '{}'.format(request.to_hash[:8]),
                           '^{}'.format(request.from_hash[:8]))
    for commit_str in re.split("{}\n?".format(separator), commits):
        if re.search(MERGE_PATTERN_SEARCH_TO_SKIP, commit_str) is not None:
            logger.info(
                "Found merge commit which skip further processing. Pattern {}".format(MERGE_PATTERN_SEARCH_TO_SKIP))
            logger.info(commit_str)
            break
        stripped_commit_msg = commit_str.strip()
        if stripped_commit_msg == '':
            continue

        logger.info("Parsed msg:\n{}".format(stripped_commit_msg))

        search = re.findall(ISSUE_TRACKER_PATTERN, commit_str)
        if len(search) > 0:
            for issue in search:
                included_issues.add(issue)

    return included_issues



import logging

from tracker.core.connectors.webhook_parser import WebHookDataParser, ParseError, RefChangeRequest

logger = logging.getLogger("webhook-parser")


class BitBucketHookParser(WebHookDataParser):

    def parse(self, request) -> RefChangeRequest:
        if len(request["changes"]) == 0:
            logger.info("Nothing was changed")
            raise ParseError()

        event_key = request["eventKey"]
        link = list(filter(lambda x: x["name"] == "http", request["repository"]["links"]["clone"]))[0]["href"]
        changes = request["changes"][0]
        type = changes["type"]
        to_hash = changes["toHash"]
        from_hash = changes["fromHash"]
        ref_id = changes["refId"].replace("refs/heads", "refs/remotes/origin")

        logger.info("Parsed request: {} {} {} {} {} {}".format(event_key, link, type, to_hash, from_hash, ref_id))
        ref_request = RefChangeRequest(
            repo_name=request["repository"]["name"],
            repo_link=link,
            update_type=type,
            to_hash=to_hash,
            from_hash=from_hash,
            ref_id=ref_id
        )
        return ref_request



import logging
import logging.config
import os

import yaml


def setup_logging(
        default_path='/logging.yaml',
        default_level=logging.INFO,
        env_key='LOG_CFG'
):
    """Setup logging configuration\n
    To override path on env define LOG_CFG properties with absolute path
    """
    path = os.getcwd() + default_path
    value = os.getenv(env_key, None)
    if value:
        path = value
    if os.path.exists(path):
        with open(path, 'rt') as f:
            config = yaml.safe_load(f.read())
        logging.config.dictConfig(config)
    else:
        logging.basicConfig(level=default_level)


setup_logging()



from abc import abstractmethod, ABC


class RefChangeRequest:
    def __init__(self, repo_name: str, repo_link: str, update_type: str, to_hash: str, from_hash: str, ref_id: str):
        self.repo_name = repo_name
        self.repo_link = repo_link
        self.update_type = update_type
        self.to_hash = to_hash
        self.from_hash = from_hash
        self.ref_id = ref_id


class WebHookDataParser(ABC):
    def __init__(self):
        pass

    @abstractmethod
    def parse(self, request) -> RefChangeRequest:
        pass


class ParseError(Exception):
    pass



import getopt
import sys
from flask import Flask
from flask_injector import FlaskInjector

from tracker.dependency import configure
from tracker.core.git_push_hook import account_api as git

app = Flask(__name__)
app.register_blueprint(git)

FlaskInjector(app=app, modules=[configure])

# Start the Flask server
if __name__ == "__main__":
    iport = 5000
    ihost = '127.0.0.1'

    myopts, args = getopt.getopt(sys.argv[1:], "p:h")

    ###############################
    # o == option
    # a == argument passed to the o
    ###############################
    for o, a in myopts:
        if o == '-p':
            iport = int(a)
        elif o == '-h':
            ihost = a
        else:
            print("Usage: %s -p port -h host" % sys.argv[0])

    app.run(host=ihost, port=iport)



import logging
import re
from concurrent.futures import ThreadPoolExecutor

from flask import Blueprint, request, make_response
from injector import inject
from jsonpickle import json

from tracker.core.connectors.issue_handler import IssueHandler
from tracker.core.connectors.webhook_parser import WebHookDataParser, ParseError
from tracker.core.fix_version_detection import process_hook_data
from tracker.env import TRACKED_BRANCH_REGEXP, WHITE_LISTED_REPOS

account_api = Blueprint('git_ba408479325a51ea5078cb427422f490df19dfb0', __name__)
logger = logging.getLogger("git:push:hook")
__git_pool__ = ThreadPoolExecutor(1)


@inject
@account_api.route("/git/ba408479325a51ea5078cb427422f490df19dfb0", methods=["POST"])
def git_push_hook(webhook_parser: WebHookDataParser, handler: IssueHandler):
    data = request.data
    if len(data) == 0:
        return make_response("", 200)

    event_request = json.loads(data)
    logger.info("Request {}".format(json.dumps(event_request)))
    try:
        ref = webhook_parser.parse(event_request)
    except ParseError:
        return make_response("", 200)

    search = re.search(TRACKED_BRANCH_REGEXP, ref.ref_id)
    if (ref.update_type == "UPDATE") and (search is not None) and (re.search(WHITE_LISTED_REPOS, ref.repo_name)):
        logger.info("Submit commit for processing")
        __git_pool__.submit(process_hook_data, ref, handler)
    else:
        logger.info("Either repo {} or branch {} is not tracked".format(ref.repo_name, ref.ref_id))

    return make_response("", 200)



import os


GIT_USER_PASS = {
    "username": os.environ["GIT_USERNAME"],
    "token": os.environ["GIT_TOKEN"]
}

# Git properties
TRACKED_BRANCH_REGEXP = os.getenv("TRACKED_BRANCH_REGEXP", "(release/.*|hotfix/.*|support/.*|develop|dev)")
MERGE_PATTERN_SEARCH_TO_SKIP = os.getenv("MERGE_PATTERN_SEARCH_TO_SKIP", "Merge.*((release\/|support\/|hotfix\/)|(tag)).*(develop|dev).*")
WHITE_LISTED_REPOS = os.getenv("WHITE_LISTED_REPOS", ".*")

# Issue tracker properties
ISSUE_TRACKER_PATTERN = os.getenv("ISSUE_TRACKER_PATTERN")







import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("VERSION", "r") as fh:
    version = fh.read().strip()

setuptools.setup(
    name="git-issue-tracker",
    version=version,
    author="Aleksandr Belyaev",
    author_email="lexbel89@gmail.com",
    description="Git issue tracker",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/lexbel/git_issue_tracker",
    packages=setuptools.find_packages(),
    install_requires=[
        "injector==0.18.3",
        "Flask==1.1.1",
        "Flask-Injector==0.12.3",
        "GitPython==3.1.0"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
